@@ -12,22 +12,28 @@
 //! Values are provided as [Stream]s of updates that can be subscribed to as
 //! necessary throughout your application.
 //!
-//! # Examples
+//! <br>
+//!
+//! # Usage
+//!
+//! Add `async-injector` to your `Cargo.toml`.
+//!
+//! ```toml
+//! [dependencies]
+//! async-injector = "0.18.1"
+//! ```
 //!
 //! In the following we'll showcase the injection of a *fake* `Database`. The
 //! idea here would be that if something about the database connection changes,
 //! a new instance of `Database` would be created and cause the application to
 //! update.
 //!
-//! > This is available as the `fake_database` example and you can run it
-//! > yourself using:
+//! > This is available as the `fake_database` example:
 //! > ```sh
 //! > cargo run --example fake_database
 //! > ```
 //!
 //! ```rust
-//! use tokio::time;
-//!
 //! #[derive(Clone)]
 //! struct Database;
 //!
@@ -37,11 +43,10 @@
 //!     let (mut database_stream, mut database) = injector.stream::<Database>().await;
 //!
 //!     // Insert the database dependency in a different task in the background.
-//!     tokio::spawn({
+//!     let _ = tokio::spawn({
 //!         let injector = injector.clone();
 //!
 //!         async move {
-//!             time::sleep(time::Duration::from_secs(2)).await;
 //!             injector.update(Database).await;
 //!         }
 //!     });
@@ -60,6 +65,8 @@
 //! With a bit of glue, this means that your application can be reconfigured
 //! without restarting it. Providing a richer user experience.
 //!
+//! <br>
+//!
 //! ## Injecting multiple things of the same type
 //!
 //! In the previous section you might've noticed that the injected value was
@@ -68,8 +75,12 @@
 //! This can be useful when dealing with overly generic types like [String].
 //!
 //! The tag used must be serializable with [serde]. It must also not use any
-//! components which [cannot be hashed], like `f32` and `f64`. This will
-//! otherwise cause an error to be raised as it's being injected.
+//! components which [cannot be hashed], like `f32` and `f64`.
+//!
+//! > This is available as the `ticker` example:
+//! > ```sh
+//! > cargo run --example ticker
+//! > ```
 //!
 //! ```rust,no_run
 //! use async_injector::Key;
@@ -138,24 +149,33 @@
 //! }
 //! ```
 //!
-//! # The `Provider` derive
+//! <br>
+//!
+//! ## The `Provider` derive
 //!
 //! The following showcases how the [Provider] derive can be used to
-//! conveniently wait for groups of dependencies to become available.
+//! conveniently wait for groups of dependencies to be supplied.
 //!
-//! Below we're waiting for two database parameters to become updated:
-//! * `url`
-//! * `connection_limit`
+//! Below we're waiting for two database parameters to become updated: `url` and
+//! `connection_limit`.
 //!
-//! Also note how they're being update asynchronously in a background thread to
-//! simulate being updated "somewhere else". This could be an update event
-//! caused by a multitude of things, like a configuration change in a frontend.
+//! Note how the update happens in a background thread to simulate it being
+//! supplied "somewhere else". In the real world this could be caused by a
+//! multitude of things, like a configuration change in a frontend.
 //!
-//! ```rust,no_run
-//! use async_injector::{Key, Provider, Injector};
+//! > This is available as the `provider` example:
+//! > ```sh
+//! > cargo run --example provider
+//! > ```
+//!
+//! ```rust
+//! use async_injector::{Injector, Key, Provider};
 //! use serde::Serialize;
 //! use std::error::Error;
+//! use std::future::pending;
 //! use std::time::Duration;
+//! use tokio::task::yield_now;
+//! use tokio::time::sleep;
 //!
 //! /// Fake database connection.
 //! #[derive(Clone, Debug, PartialEq, Eq)]
@@ -169,6 +189,7 @@
 //! pub enum Tag {
 //!     DatabaseUrl,
 //!     ConnectionLimit,
+//!     Shutdown,
 //! }
 //!
 //! /// A group of database params to wait for until they become available.
@@ -180,59 +201,90 @@
 //!     connection_limit: u32,
 //! }
 //!
-//! async fn update_db_params(injector: Injector, db_url: Key<String>, connection_limit: Key<u32>) {
-//!     tokio::time::sleep(Duration::from_secs(2));
+//! async fn update_db_params(
+//!     injector: Injector,
+//!     db_url: Key<String>,
+//!     connection_limit: Key<u32>,
+//!     shutdown: Key<bool>,
+//! ) {
+//!     injector
+//!         .update_key(&db_url, String::from("example.com"))
+//!         .await;
 //!
-//!     injector.update_key(&db_url, String::from("example.com")).await;
-//!     injector.update_key(&connection_limit, 5).await;
+//!     for limit in 5..10 {
+//!         sleep(Duration::from_millis(100)).await;
+//!         injector.update_key(&connection_limit, limit).await;
+//!     }
+//!
+//!     // Yield to give the update a chance to propagate.
+//!     yield_now().await;
+//!     injector.update_key(&shutdown, true).await;
 //! }
 //!
 //! /// Fake service that runs for two seconds with a configured database.
-//! async fn service(db: Database) {
-//!     tokio::time::sleep(Duration::from_secs(2));
+//! async fn service(database: Database) {
+//!     println!("Starting new service with database: {:?}", database);
+//!     pending::<()>().await;
 //! }
 //!
-//! #[tokio::test]
-//! async fn test_provider() -> Result<(), Box<dyn Error>> {
+//! #[tokio::main]
+//! async fn main() -> Result<(), Box<dyn Error>> {
 //!     let db_url = Key::<String>::tagged(Tag::DatabaseUrl)?;
 //!     let connection_limit = Key::<u32>::tagged(Tag::ConnectionLimit)?;
+//!     let shutdown = Key::<bool>::tagged(Tag::Shutdown)?;
 //!
 //!     let injector = Injector::new();
 //!
-//!     /// Set up asynchronous task that updates the parameters in the background.
-//!     tokio::spawn(update_db_params(injector.clone(), db_url, connection_limit));
+//!     // Set up asynchronous task that updates the parameters in the background.
+//!     tokio::spawn(update_db_params(
+//!         injector.clone(),
+//!         db_url,
+//!         connection_limit,
+//!         shutdown.clone(),
+//!     ));
 //!
-//!     let provider = DatabaseParams::new(&injector).await?;
+//!     let mut provider = DatabaseParams::provider(&injector).await?;
+//!
+//!     // Wait until database is configured.
+//!     let params = provider.wait().await;
+//!
+//!     let database = Database {
+//!         url: params.url,
+//!         connection_limit: params.connection_limit,
+//!     };
+//!
+//!     assert_eq!(
+//!         database,
+//!         Database {
+//!             url: String::from("example.com"),
+//!             connection_limit: 5
+//!         }
+//!     );
+//!
+//!     let (mut shutdown, is_shutdown) = injector.stream_key(&shutdown).await;
+//!
+//!     if is_shutdown == Some(true) {
+//!         return Ok(());
+//!     }
+//!
+//!     let fut = service(database);
+//!     tokio::pin!(fut);
 //!
 //!     loop {
-//!         /// Wait until database is configured.
-//!         let database = loop {
-//!             if let Some(update) = provider.update().await {
-//!                 break Database {
-//!                     url: update.url,
-//!                     connection_limit: update.connection_limit,
-//!                 };
+//!         tokio::select! {
+//!             _ = &mut fut => {
+//!                 break;
 //!             }
-//!         };
-//!
-//!         assert_eq!(
-//!             new_database,
-//!             Database {
-//!                 url: String::from("example.com"),
-//!                 connection_limit: 5
-//!             }
-//!         );
-//!
-//!         loop {
-//!             tokio::select! {
-//!                 _ = service(database.clone()) => {
+//!             is_shutdown = shutdown.recv() => {
+//!                 if is_shutdown == Some(true) {
 //!                     break;
 //!                 }
-//!                 update = provider.update().await {
-//!                     match update {
-//!                         None => break,
-//!                     }
-//!                 }
+//!             }
+//!             params = provider.wait() => {
+//!                 fut.set(service(Database {
+//!                     url: params.url,
+//!                     connection_limit: params.connection_limit,
+//!                 }));
 //!             }
 //!         }
 //!     }
@@ -244,9 +296,10 @@
 //! [cannot be hashed]: https://internals.rust-lang.org/t/f32-f64-should-implement-hash/5436
 //! [Injector]: https://docs.rs/async-injector/0/async_injector/struct.Injector.html
 //! [Key]: https://docs.rs/async-injector/0/async_injector/struct.Key.html
-//! [Provider]: https://docs.rs/async-injector-derive/0/async_injector_derive/derive.Provider.html
+//! [Provider]: https://docs.rs/async-injector/0/async_injector/derive.Provider.html
 //! [serde]: https://serde.rs
 //! [Stream]: https://docs.rs/futures-core/0/futures_core/stream/trait.Stream.html
+//! [String]: https://doc.rust-lang.org/std/string/struct.String.html
 
 #![deny(missing_docs)]
 
@@ -272,11 +325,8 @@ pub mod derive {
     pub use tokio::select;
 }
 
-#[macro_use]
-#[allow(unused_imports)]
-extern crate async_injector_derive;
-#[doc(hidden)]
-pub use self::async_injector_derive::*;
+#[doc(inline)]
+pub use async_injector_derive::Provider;
 
 /// Errors that can be raised by various functions in the [Injector].
 #[derive(Debug)]
@@ -454,10 +504,7 @@ struct Inner {
 /// An injector of dependencies.
 ///
 /// Injectors are defined in hierarchies where an injector is either the root
-/// injector as created using [setup] or [setup_with_driver], or the child of
-/// another injector through [Injector::child].
-///
-/// Child injectors receive all the updates of their ancestors.
+/// injector as created using [Injector::new].
 #[derive(Clone)]
 pub struct Injector {
     inner: Arc<Inner>,
@@ -468,7 +515,7 @@ impl Injector {
     ///
     /// # Example
     ///
-    /// ```rust
+    /// ```
     /// #[tokio::main]
     /// async fn main() {
     ///     let injector = async_injector::Injector::new();
@@ -483,7 +530,7 @@ impl Injector {
     ///
     /// # Example using a [Stream]
     ///
-    /// ```rust
+    /// ```
     /// use std::error::Error;
     ///
     /// #[derive(Clone)]
@@ -502,11 +549,7 @@ impl Injector {
     /// }
     /// ```
     pub fn new() -> Self {
-        Self {
-            inner: Arc::new(Inner {
-                storage: Default::default(),
-            }),
-        }
+        Self::default()
     }
 
     /// Get a value from the injector.
@@ -519,7 +562,7 @@ impl Injector {
     ///
     /// # Examples
     ///
-    /// ```rust
+    /// ```
     /// #[tokio::main]
     /// async fn main() {
     ///     let injector = async_injector::Injector::new();
@@ -548,7 +591,7 @@ impl Injector {
     ///
     /// # Examples
     ///
-    /// ```rust
+    /// ```
     /// use async_injector::{Key, Injector};
     /// use std::error::Error;
     ///
@@ -589,7 +632,7 @@ impl Injector {
     ///
     /// # Examples
     ///
-    /// ```rust
+    /// ```
     /// #[tokio::main]
     /// async fn main() {
     ///     let injector = async_injector::Injector::new();
@@ -616,7 +659,7 @@ impl Injector {
     ///
     /// # Examples
     ///
-    /// ```rust
+    /// ```
     /// use async_injector::{Key, Injector};
     /// use std::error::Error;
     ///
@@ -637,7 +680,7 @@ impl Injector {
         T: Clone + Any + Send + Sync,
     {
         let key = key.as_ref().as_raw_key();
-        let value = Value::new(T::from(value));
+        let value = Value::new(value);
 
         let mut storage = self.inner.storage.write().await;
         let storage = storage.entry(key.clone()).or_default();
@@ -650,7 +693,7 @@ impl Injector {
     ///
     /// # Examples
     ///
-    /// ```rust
+    /// ```
     /// #[tokio::main]
     /// async fn main() {
     ///     let injector = async_injector::Injector::new();
@@ -671,7 +714,7 @@ impl Injector {
     ///
     /// # Examples
     ///
-    /// ```rust
+    /// ```
     /// use async_injector::{Key, Injector};
     /// use std::error::Error;
     ///
@@ -705,7 +748,7 @@ impl Injector {
     ///
     /// # Examples
     ///
-    /// ```rust
+    /// ```
     /// #[tokio::main]
     /// async fn main() {
     ///     let injector = async_injector::Injector::new();
@@ -733,7 +776,7 @@ impl Injector {
     ///
     /// # Examples
     ///
-    /// ```rust
+    /// ```
     /// use async_injector::{Key, Injector};
     /// use std::error::Error;
     ///
@@ -783,7 +826,7 @@ impl Injector {
     ///
     /// # Examples
     ///
-    /// ```rust
+    /// ```
     /// #[tokio::main]
     /// async fn main() {
     ///     let injector = async_injector::Injector::new();
@@ -804,7 +847,7 @@ impl Injector {
     ///
     /// # Examples
     ///
-    /// ```rust
+    /// ```
     /// use async_injector::{Injector, Key};
     /// use std::error::Error;
     ///
@@ -840,20 +883,17 @@ impl Injector {
 
         let value = storage.value.read().await;
 
-        if let Some(value) = &*value {
-            // Safety: The expected type parameter is encoded and
-            // maintained in the Stream type.
-            Some(unsafe { value.downcast_ref::<T>().clone() })
-        } else {
-            None
-        }
+        value.as_ref().map(|value| {
+            // Safety: Ref<T> instances can only be produced by checked fns.
+            unsafe { value.downcast_ref::<T>().clone() }
+        })
     }
 
     /// Get an existing value and setup a stream for updates at the same time.
     ///
     /// # Examples
     ///
-    /// ```rust
+    /// ```
     /// use std::error::Error;
     ///
     /// #[derive(Debug, Clone, PartialEq, Eq)]
@@ -893,7 +933,7 @@ impl Injector {
     ///
     /// # Examples
     ///
-    /// ```rust
+    /// ```
     /// use async_injector::{Injector, Key};
     /// use std::error::Error;
     ///
@@ -939,14 +979,11 @@ impl Injector {
 
         let value = storage.value.read().await;
 
-        let value = match &*value {
-            Some(value) => {
-                // Safety: The expected type parameter is encoded and
-                // maintained in the Stream type.
-                Some(unsafe { value.downcast_ref::<T>().clone() })
-            }
-            None => None,
-        };
+        let value = value.as_ref().map(|value| {
+            // Safety: The expected type parameter is encoded and maintained
+            // in the Stream type.
+            unsafe { value.downcast_ref::<T>().clone() }
+        });
 
         let stream = Stream {
             rx,
@@ -960,7 +997,7 @@ impl Injector {
     ///
     /// # Examples
     ///
-    /// ```rust
+    /// ```
     /// use std::error::Error;
     ///
     /// #[derive(Clone)]
@@ -989,7 +1026,7 @@ impl Injector {
     ///
     /// # Examples
     ///
-    /// ```rust
+    /// ```
     /// use async_injector::{Injector, Key};
     /// use std::error::Error;
     ///
@@ -1025,6 +1062,16 @@ impl Injector {
     }
 }
 
+impl Default for Injector {
+    fn default() -> Self {
+        Self {
+            inner: Arc::new(Inner {
+                storage: Default::default(),
+            }),
+        }
+    }
+}
+
 #[derive(Debug, Clone, PartialOrd, Ord, PartialEq, Eq, Hash)]
 struct RawKey {
     type_id: TypeId,
@@ -1115,7 +1162,7 @@ where
     ///
     /// # Examples
     ///
-    /// ```rust
+    /// ```
     /// use async_injector::Key;
     ///
     /// struct Foo;
@@ -1133,7 +1180,7 @@ where
     ///
     /// # Examples
     ///
-    /// ```rust
+    /// ```
     /// use serde::Serialize;
     /// use async_injector::Key;
     /// use std::error::Error;
@@ -1207,7 +1254,7 @@ where
     ///
     /// # Examples
     ///
-    /// ```rust
+    /// ```
     /// use std::error::Error;
     ///
     /// #[derive(Clone)]
@@ -1229,11 +1276,11 @@ where
         let value = self.value.read().await;
 
         let result = RefReadGuard::try_map(value, |value| {
-            match value {
-                // Safety: Ref<T> instances can only be produced by checked fns.
-                Some(value) => Some(unsafe { value.downcast_ref::<T>() }),
-                None => None,
-            }
+            value.as_ref().map(|value| {
+                // Safety: The expected type parameter is encoded and
+                // maintained in the Stream type.
+                unsafe { value.downcast_ref::<T>() }
+            })
         });
 
         result.ok()
@@ -1244,7 +1291,7 @@ where
     ///
     /// # Examples
     ///
-    /// ```rust
+    /// ```
     /// use std::error::Error;
     ///
     /// #[derive(Clone)]
@@ -1265,11 +1312,11 @@ where
     pub async fn load(&self) -> Option<T> {
         let value = self.value.read().await;
 
-        match &*value {
-            // Safety: Ref<T> instances can only be produced by checked fns.
-            Some(value) => Some(unsafe { value.downcast_ref::<T>().clone() }),
-            None => None,
-        }
+        value.as_ref().map(|value| {
+            // Safety: The expected type parameter is encoded and maintained
+            // in the Stream type.
+            unsafe { value.downcast_ref::<T>().clone() }
+        })
     }
 }
 
